/* eslint-disable no-trailing-spaces */
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames/bind';
import SearchResultContentWrapper from './SearchResultContentWrapper';
import { numSignHelper } from '../../utils/num';
import {
  hotTagsType,
  hotTradesType,
  hotDriversType,
  recentSearchType,
  tradeResultType,
  newsResultType,
} from '../../utils/propTypes';
import oldDriverIcon from '../../assets/icon-default-driver.svg';
import styles from './AnueSearchRealtimeResult.scss';
import LazyImage from '../LazyImage/LazyImage';
import AnueLink from '../../utils/anueLink';
import { getCategory, getExchange } from '../../utils/anueSearch';
import { driverNameEllipsisHelper, newsPublishedDateHelper } from './helpers';

const cx = classnames.bind(styles);
const anueUtilLink = new AnueLink('prod');
const MAX_RECENT_SEARCH = 5;
const MAX_SEARCH_NEWS = 2;

function AnueSearchRealtimeResult({
  hotTags,
  hotTrades,
  hotDrivers,
  recentSearch,
  tradeResult,
  newsResult,
  driversResult,
  searchString,
  customWrapperStyles,
  onHotTagClick,
  onRecentSearchRowClick,
  onRecentSearchRowRemoveClick,
  onRecentSearchRemoveAllClick,
  onResultSearchRowClick,
  searchResultPageLink,
}) {
  const defaultSearchPageLink = anueUtilLink.generateSearchResultLink({
    keyword: searchString,
  });
  const searchPageLink = searchResultPageLink || defaultSearchPageLink;
  const getCode = symbol => {
    const symbolSplit = String(symbol).split(':');

    if (!symbolSplit || symbolSplit.length < 2) {
      return null;
    }

    return symbolSplit[1];
  };
  const renderHotTags = () => (
    // Display nothing if receive a empty array
    <SearchResultContentWrapper isLoaded={hotTags && hotTags.length > 0}>
      <div className={cx('hot-tags')}>
        <div>
          <p>熱搜：</p>
        </div>
        <div>
          {hotTags &&
            hotTags.map(tag => {
              const handleClick = () => onHotTagClick(tag);

              return (
                <a key={tag.tagName} href={tag.link} onClick={handleClick}>
                  {tag.tagName}
                </a>
              );
            })}
        </div>
      </div>
    </SearchResultContentWrapper>
  );
  const renderHotTrades = () => (
    // Display nothing if receive a empty array
    <SearchResultContentWrapper title="熱門行情" isLoaded={hotTrades && hotTrades.length > 0}>
      <div className={cx('content-rows')}>
        {hotTrades &&
          hotTrades.map((trade, i) => {
            const code = getCode(trade.symbol);
            const display = trade.displayName || code;

            return display && trade.link ? (
              // eslint-disable-next-line react/no-array-index-key
              <a key={`hot-trade${i}`} className={cx('row', 'icon--hot')} href={trade.link}>
                {display}
              </a>
            ) : null;
          })}
      </div>
    </SearchResultContentWrapper>
  );
  const renderRecentSearch = () => (
    // Display nothing if receive a empty array
    <SearchResultContentWrapper
      title="最近搜尋"
      headerActionName="全部刪除"
      isLoaded={recentSearch && recentSearch.length > 0}
      onHeaderActionClick={onRecentSearchRemoveAllClick}
    >
      <div className={cx('content-rows')}>
        {recentSearch &&
          recentSearch.slice(0, MAX_RECENT_SEARCH).map((search, i) => {
            const handleClickRow = () => onRecentSearchRowClick(search.keyword);
            const handleClickRemove = () => onRecentSearchRowRemoveClick(search.keyword);

            return (
              // eslint-disable-next-line react/no-array-index-key
              <div key={`recent-search${i}`} className={cx('row', 'limit-length', 'icon--time')}>
                <a onClick={handleClickRow}>{search.keyword}</a>
                <p className={cx('icon-cross')} onClick={handleClickRemove} />
              </div>
            );
          })}
      </div>
    </SearchResultContentWrapper>
  );
  const renderDrivers = (driverList, isSearchResult) => (
    // isSearchResult ? Display the default message if receive a empty array : // Display nothing if receive a empty array
    <SearchResultContentWrapper
      title="基金老司機"
      isLoaded={isSearchResult ? driverList : driverList && driverList.length > 0}
      isCols
    >
      <div className={cx('content-cols')}>
        {driverList && driverList.length > 0 ? (
          driverList.map((driver, i) => (
            <a
              // eslint-disable-next-line react/no-array-index-key
              key={`driver${i}`}
              className={cx('col')}
              href={driver.link}
              onClick={isSearchResult ? onResultSearchRowClick : null}
            >
              <div className={cx('avatar')}>
                <LazyImage isRounded src={driver.avatarPicture || oldDriverIcon} />
              </div>
              <p
                className={cx('mark', 'name')}
                dangerouslySetInnerHTML={{
                  __html: driverNameEllipsisHelper(driver.driverName),
                }}
              />
              <span className={cx('sub-title')}>一年績效</span>
              <span
                className={cx('perf', {
                  red: driver.return1year > 0,
                  green: driver.return1year < 0,
                })}
              >
                {driver.return1year != null
                  ? numSignHelper(driver.return1year.toFixed(2), {
                      isPercentage: true,
                    })
                  : '----'}
              </span>
            </a>
          ))
        ) : (
          <div className={cx('empty')}>沒有符合搜尋條件的資料</div>
        )}
      </div>
    </SearchResultContentWrapper>
  );
  const renderSearchTradeResult = () => (
    // Display the default message if receive a empty array.
    <SearchResultContentWrapper title="行情" isLoaded={tradeResult}>
      <div className={cx('content-rows')}>
        {tradeResult && tradeResult.length > 0 ? (
          tradeResult.map((trade, i) => {
            const category = getCategory(trade);
            const exchange = getExchange(trade);
            const info = [category, exchange].filter(v => !!v).join('-');

            return (
              // eslint-disable-next-line react/no-array-index-key
              <a key={`trade${i}`} className={cx('row', 'result')} href={trade.link} onClick={onResultSearchRowClick}>
                <p className={cx('title')}>
                  <span className={cx('mark')} dangerouslySetInnerHTML={{ __html: trade.code }} />
                  <small>{info}</small>
                </p>
                <p className={cx('summary')}>
                  <span className={cx('mark')} dangerouslySetInnerHTML={{ __html: trade.chName }} />
                  <span className={cx('mark')} dangerouslySetInnerHTML={{ __html: trade.enName }} />
                </p>
              </a>
            );
          })
        ) : (
          <div className={cx('empty')}>沒有符合搜尋條件的資料</div>
        )}
      </div>
    </SearchResultContentWrapper>
  );
  const renderSearchNewsResult = () => (
    // Display the default message if receive a empty array.
    <SearchResultContentWrapper title="新聞" isLoaded={newsResult}>
      <div className={cx('content-rows')}>
        {newsResult && newsResult.length > 0 ? (
          newsResult.slice(0, MAX_SEARCH_NEWS).map((n, i) => (
            // eslint-disable-next-line react/no-array-index-key
            <a key={`news${i}`} className={cx('row', 'result')} href={n.link} onClick={onResultSearchRowClick}>
              <p className={cx('title')}>
                <span className={cx('mark')} dangerouslySetInnerHTML={{ __html: n.title }} />
              </p>
              <p className={cx('summary')}>
                <span>{newsPublishedDateHelper(n.publishAt)}</span>
                <span>{n.summary}</span>
              </p>
            </a>
          ))
        ) : (
          <div className={cx('empty')}>沒有符合搜尋條件的資料</div>
        )}
      </div>
    </SearchResultContentWrapper>
  );

  return (
    <div className={cx('anue-search-result')} style={customWrapperStyles}>
      {searchString ? (
        <div className={cx('search-sections', 'section')}>
          <a className={cx('current-search')} href={searchPageLink} onClick={onResultSearchRowClick}>
            查看「{searchString}」的所有搜尋結果＞
          </a>
          {renderSearchTradeResult()}
          {renderSearchNewsResult()}
          {renderDrivers(driversResult, true)}
        </div>
      ) : (
        <div className={cx('hot-sections', 'section')}>
          {renderHotTags()}
          {renderHotTrades()}
          {renderRecentSearch()}
          {renderDrivers(hotDrivers)}
        </div>
      )}
    </div>
  );
}

AnueSearchRealtimeResult.propTypes = {
  hotTags: PropTypes.arrayOf(hotTagsType),
  hotTrades: PropTypes.arrayOf(hotTradesType),
  hotDrivers: PropTypes.arrayOf(hotDriversType),
  recentSearch: PropTypes.arrayOf(recentSearchType),
  tradeResult: PropTypes.arrayOf(tradeResultType),
  newsResult: PropTypes.arrayOf(newsResultType),
  driversResult: PropTypes.arrayOf(hotDriversType),
  searchString: PropTypes.string,
  customWrapperStyles: PropTypes.objectOf(PropTypes.any),
  onHotTagClick: PropTypes.func,
  onRecentSearchRowClick: PropTypes.func,
  onRecentSearchRowRemoveClick: PropTypes.func,
  onRecentSearchRemoveAllClick: PropTypes.func,
  onResultSearchRowClick: PropTypes.func,
  searchResultPageLink: PropTypes.string,
};

AnueSearchRealtimeResult.defaultProps = {
  hotTags: null,
  hotTrades: null,
  hotDrivers: null,
  recentSearch: null,
  tradeResult: null,
  newsResult: null,
  driversResult: null,
  searchTradeResult: null,
  searchDriverResult: null,
  searchNewsResult: null,
  searchString: null,
  customWrapperStyles: null,
  searchResultPageLink: null,
  onHotTagClick: () => {},
  onRecentSearchRowClick: () => {},
  onRecentSearchRowRemoveClick: () => {},
  onRecentSearchRemoveAllClick: () => {},
  onResultSearchRowClick: () => {},
};

export default AnueSearchRealtimeResult;
